<?php
session_start();
define('OFFLINE_THRESHOLD_SECONDS', 60);

// --- Verificaciones Iniciales y Obtención de Datos ---
if (!isset($_SESSION['user_id'])) {
    header('Location: login.php');
    exit();
}
$user_id = $_SESSION['user_id'];
$current_user_name = $_SESSION['name'] ?? 'Usuario';
$current_user_phone = $_SESSION['phone'] ?? '';
if (!isset($_GET['id']) || !is_numeric($_GET['id']) || intval($_GET['id']) <= 0) {
    error_log("Invalid chat ID requested by user: " . $user_id);
    header('Location: dashboard.php?error=invalid_chat_id');
    exit();
}
$other_user_id = intval($_GET['id']);
if ($user_id === $other_user_id) {
    header('Location: dashboard.php?error=self_chat');
    exit();
}

require_once 'config.php';

// --- Obtener Información del Otro Usuario y su Estado ---
$other_user_status = 'offline';
$other_user_last_activity = null;
$other_user_name = 'Desconocido';
$other_user_phone = '';
$user_query = "SELECT u.id, u.name, u.phone, us.status, us.last_activity 
              FROM users u 
              LEFT JOIN user_status us ON u.id = us.user_id 
              WHERE u.id = ?";
$user_stmt = $conn->prepare($user_query);
if ($user_stmt) {
    $user_stmt->bind_param("i", $other_user_id);
    $user_stmt->execute();
    $user_result = $user_stmt->get_result();
    if ($user_result->num_rows > 0) {
        $other_user = $user_result->fetch_assoc();
        $other_user_name = $other_user['name'];
        $other_user_phone = $other_user['phone'];
        $db_status = $other_user['status'];
        $other_user_last_activity = $other_user['last_activity'];
        if ($db_status === 'online' && $other_user_last_activity) {
            $last_activity_timestamp = strtotime($other_user_last_activity);
            if ((time() - $last_activity_timestamp) <= OFFLINE_THRESHOLD_SECONDS) {
                $other_user_status = 'online';
            } else {
                $other_user_status = 'offline';
            }
        } else {
            $other_user_status = 'offline';
        }
    } else {
        $user_stmt->close();
        $conn->close();
        error_log("Chat user ID (id={$other_user_id}) not found for user: {$user_id}.");
        header('Location: dashboard.php?error=chat_user_not_found');
        exit();
    }
    $user_stmt->close();
} else {
    error_log("Error preparing user info statement: " . $conn->error);
    $conn->close();
    header('Location: dashboard.php?error=db_error');
    exit();
}

// --- Obtener Historial de Mensajes ---
$messages_query = "SELECT id, sender_id, receiver_id, message, type, file_url, file_name, file_size, file_type, timestamp, `read`, delivered 
                   FROM messages 
                   WHERE ((sender_id = ? AND receiver_id = ?) OR (sender_id = ? AND receiver_id = ?))
                     AND deleted = 0 
                   ORDER BY timestamp ASC";
$messages = [];
$messages_stmt = $conn->prepare($messages_query);
if ($messages_stmt) {
    $messages_stmt->bind_param("iiii", $user_id, $other_user_id, $other_user_id, $user_id);
    $messages_stmt->execute();
    $messages_result = $messages_stmt->get_result();
    while ($row = $messages_result->fetch_assoc()) {
        // Formatear timestamp a ISO 8601 UTC para JavaScript
        if ($row['timestamp']) {
            try {
                $date = new DateTime($row['timestamp'], new DateTimeZone('UTC')); 
                $row['timestamp_iso'] = $date->format('c');
            } catch (Exception $e) {
                error_log("Invalid timestamp DB ID {$row['id']}: {$row['timestamp']}. Error: " . $e->getMessage());
                $row['timestamp_iso'] = null;
            }
        } else {
            $row['timestamp_iso'] = null;
        }
        $messages[] = $row;
    }
    $messages_stmt->close();
} else {
    error_log("Error preparing messages history: " . $conn->error);
}

// --- Marcar Mensajes como Leídos ---
$update_read_on_load_query = "UPDATE messages SET `read` = 1 WHERE sender_id = ? AND receiver_id = ? AND `read` = 0";
$update_stmt = $conn->prepare($update_read_on_load_query);
if ($update_stmt) {
    $update_stmt->bind_param("ii", $other_user_id, $user_id);
    $update_stmt->execute();
    $update_stmt->close();
} else {
    error_log("Error preparing update read on load: " . $conn->error);
}

// --- Actualizar Estado Propio ---
$update_my_status_query = "INSERT INTO user_status (user_id, status, last_activity) VALUES (?, 'online', NOW())
                            ON DUPLICATE KEY UPDATE status = 'online', last_activity = NOW()";
$update_my_stmt = $conn->prepare($update_my_status_query);
if ($update_my_stmt) {
    $update_my_stmt->bind_param("i", $user_id);
    $update_my_stmt->execute();
    $update_my_stmt->close();
} else {
    error_log("Error preparing update own status: " . $conn->error);
}

$conn->close();
?>
<!DOCTYPE html>
<html lang="es" class="h-full">
<head>
  <meta charset="UTF-8">
  <meta name="viewport" content="width=device-width, initial-scale=1.0, maximum-scale=1.0, user-scalable=no">
  <title>FlyChat - Chat con <?php echo htmlspecialchars($other_user_name); ?></title>

  <!-- Dependencias CSS -->
  <link href="https://cdnjs.cloudflare.com/ajax/libs/tailwindcss/2.2.19/tailwind.min.css" rel="stylesheet">
  <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/font-awesome/6.0.0-beta3/css/all.min.css">
  <link rel="stylesheet" href="https://cdn.jsdelivr.net/npm/emoji-mart@latest/css/emoji-mart.css">
  <link rel="stylesheet" href="https://cdnjs.cloudflare.com/ajax/libs/photoswipe/5.4.4/photoswipe.min.css" />

  <!-- CSS Personalizado -->
  <link rel="stylesheet" href="css/style.css">

  <!-- Se incluye PeerJS para la funcionalidad de llamadas -->
  <script src="https://unpkg.com/peerjs@1.4.7/dist/peerjs.min.js"></script>
  
  <style>
    /* Estilos base y overrides existentes */
    * { -webkit-tap-highlight-color: transparent; outline: none; }
    #chat-messages { scroll-behavior: smooth; }
    .message-status-icon { font-size: 0.75rem; opacity: 0.8; }
    #emoji-picker-container { position: absolute; bottom: calc(100% + 8px); left: 8px; z-index: 50; display: none; background-color: white; border-radius: 8px; box-shadow: 0 4px 12px rgba(0,0,0,0.15); }
    .typing-indicator span { display: inline-block; width: 6px; height: 6px; margin: 0 1px; background-color: #9ca3af; border-radius: 50%; animation: typing 1s infinite ease-in-out; }
    .typing-indicator span:nth-child(2) { animation-delay: 0.2s; }
    .typing-indicator span:nth-child(3) { animation-delay: 0.4s; }
    @keyframes typing { 0%, 100% { opacity: 0.3; transform: scale(0.8); } 50% { opacity: 1; transform: scale(1); } }
    @keyframes spin { 0% { transform: rotate(0deg); } 100% { transform: rotate(360deg); } }
    .spinner-tiny { border: 2px solid rgba(255,255,255,0.3); border-top-color: white; border-radius: 50%; width: 12px; height: 12px; animation: spin 1s linear infinite; }
    .status-dot { display: inline-block; }
    .pswp__bg { background: rgba(0, 0, 0, 0.9) !important; }
    .pswp--one-image .pswp__counter { display: none !important; }
    .pswp__button { background-color: rgba(0, 0, 0, 0.5) !important; }
    .blue-gradient { background: linear-gradient(to right, #3b82f6, #2563eb); }
    
    /* Padding inferior para mensajes */
    #chat-messages {
         padding-bottom: 4rem;
    }
    
    /* Estilos para el Modal de Llamadas Mejorado */
    #call-modal {
      display: none;
      position: fixed;
      top: 0;
      left: 0;
      width: 100%;
      height: 100%;
      background: rgba(0, 0, 0, 0.7);
      z-index: 2000;
      align-items: center;
      justify-content: center;
    }
    
    #call-modal-content {
      background: #333;
      padding: 20px;
      border-radius: 12px;
      text-align: center;
      color: #fff;
      max-width: 320px;
      width: 90%;
      box-shadow: 0 8px 24px rgba(0, 0, 0, 0.3);
    }
    
    #call-modal-avatar {
      width: 80px;
      height: 80px;
      background-color: #4a5568;
      border-radius: 50%;
      margin: 0 auto 15px;
      display: flex;
      align-items: center;
      justify-content: center;
      font-size: 32px;
      font-weight: bold;
      color: #e2e8f0;
    }
    
    #call-modal-status {
      font-size: 14px;
      color: #a0aec0;
      margin: 5px 0;
    }
    
    #call-timer {
      font-family: monospace;
      font-size: 16px;
      margin: 10px 0;
      background-color: rgba(0, 0, 0, 0.2);
      padding: 4px 12px;
      border-radius: 12px;
      display: inline-block;
      color: #90cdf4;
    }
    
    #call-modal button {
      margin: 8px 5px;
      padding: 12px 0;
      border: none;
      border-radius: 100px;
      cursor: pointer;
      width: 46px;
      height: 46px;
      display: inline-flex;
      align-items: center;
      justify-content: center;
      transition: all 0.2s ease;
    }
    
    #call-modal button:hover {
      transform: scale(1.05);
    }
    
    .btn-accept { 
      background-color: #4CAF50;
      color: white; 
      box-shadow: 0 4px 12px rgba(76, 175, 80, 0.3);
    }
    
    .btn-decline, .btn-cancel, .btn-hangup { 
      background-color: #f44336; 
      color: white; 
      box-shadow: 0 4px 12px rgba(244, 67, 54, 0.3);
    }
    
    .btn-mute {
      background-color: #4a5568;
      color: white;
    }
    
    .btn-mute.active {
      background-color: #e53e3e;
    }
    
    /* Fix para el header en Safari */
    header {
      position: sticky;
      top: 0;
      z-index: 50;
      width: 100%;
    }
    
    /* Asegurarse que todos los botones sean fácilmente tocables en móviles */
    button, a {
      cursor: pointer;
      touch-action: manipulation;
    }
    
    /* Fix específico para Safari en el contendor principal */
    #chat-container {
      position: relative;
      display: flex;
      flex-direction: column;
      width: 100%;
      height: 100%;
    }
    
    /* Estado de la conexión PeerJS */
    #peer-connection-indicator {
      position: absolute;
      top: 5px;
      right: 5px;
      font-size: 8px;
      opacity: 0.7;
      z-index: 60;
    }
  </style>
</head>
<body class="h-full bg-gray-100">

  <div id="chat-container" class="h-screen flex flex-col overflow-hidden">
    <!-- Header Fijo con z-index mejorado -->
    <header class="blue-gradient text-white p-3 md:p-4 flex items-center shadow-md flex-shrink-0 z-50">
      <a href="dashboard.php" class="mr-2 md:mr-3 text-white hover:text-blue-200 transition duration-150 flex-shrink-0">
        <i class="fas fa-arrow-left fa-lg"></i>
      </a>
      <div class="relative flex-shrink-0 mr-2 md:mr-3">
        <div class="w-9 h-9 md:w-10 md:h-10 rounded-full bg-blue-300 flex items-center justify-center text-blue-800 font-bold text-sm md:text-base">
          <span><?php echo strtoupper(substr(htmlspecialchars($other_user_name), 0, 1)); ?></span>
        </div>
        <span id="status-dot-indicator" class="status-dot absolute bottom-0 right-0 block w-3 h-3 border-2 border-white rounded-full <?php echo $other_user_status === 'online' ? 'bg-green-500' : 'bg-gray-500'; ?>"></span>
      </div>
      <div class="flex-1 min-w-0 mx-1 md:mx-2">
        <h1 class="font-semibold md:font-bold truncate text-sm md:text-base leading-tight"><?php echo htmlspecialchars($other_user_name); ?></h1>
        <p class="text-xs text-blue-100 truncate" id="status-text"><?php echo $other_user_status === 'online' ? 'Online' : 'Offline'; ?></p>
      </div>
      <div class="flex flex-shrink-0">
        <!-- Botón de historial de llamadas -->
        <button class="p-2 text-white hover:text-blue-200 transition duration-150 mr-1" id="call-history-btn" title="Historial de llamadas">
          <i class="fas fa-history"></i>
        </button>
        <!-- Botón de llamada de voz -->
        <button class="p-2 text-white hover:text-blue-200 transition duration-150" id="audio-call-btn" title="Llamada de voz">
          <i class="fas fa-phone"></i>
        </button>
      </div>
    </header>

    <!-- Área de Mensajes Scrollable -->
    <div class="flex-1 overflow-y-auto bg-gray-200 p-3 md:p-4" id="chat-messages">
      <!-- Mensajes renderizados por JS -->
    </div>

    <!-- Indicador "Escribiendo..." -->
    <div id="typing-indicator" class="px-3 md:px-4 py-1 bg-gray-200 text-gray-500 text-xs md:text-sm hidden flex-shrink-0 flex items-center">
      <span id="typing-user-name" class="mr-1"><?php echo htmlspecialchars($other_user_name); ?></span> está escribiendo
      <div class="typing-indicator inline-block ml-1"><span></span><span></span><span></span></div>
    </div>

    <!-- Área de Input Fija -->
    <div class="bg-gray-100 p-2 md:p-3 border-t border-gray-300 flex items-end flex-shrink-0">
      <div class="flex-1 bg-white rounded-full shadow-sm flex items-center px-2 py-1 relative mr-2">
        <div id="emoji-picker-container">
          <!-- Emoji Mart Picker se insertará aquí -->
        </div>
        <button class="text-gray-500 p-2 hover:text-blue-500 focus:outline-none flex-shrink-0" id="emoji-btn">
          <i class="far fa-smile fa-lg"></i>
        </button>
        <input type="text" id="message-input" placeholder="Escribe un mensaje..." class="w-full py-2 px-1 md:px-2 focus:outline-none bg-transparent text-sm md:text-base min-w-0">
        <label for="file-input" class="text-gray-500 p-2 cursor-pointer hover:text-blue-500 flex-shrink-0">
          <i class="fas fa-paperclip fa-lg"></i>
        </label>
        <input type="file" id="file-input" class="hidden" accept="image/*,video/*,application/*,text/*">
      </div>
      <button id="send-btn" class="w-10 h-10 md:w-11 md:h-11 rounded-full blue-gradient text-white flex items-center justify-center flex-shrink-0 focus:outline-none focus:ring-2 focus:ring-offset-1 focus:ring-blue-400 disabled:opacity-50 transition duration-150">
        <i class="fas fa-paper-plane"></i>
      </button>
    </div>
  </div> <!-- Fin del chat-container -->

  <!-- Modal mejorado para Llamadas -->
  <div id="call-modal">
    <div id="call-modal-content">
      <div id="call-modal-avatar">
        <span><?php echo strtoupper(substr(htmlspecialchars($other_user_name), 0, 1)); ?></span>
      </div>
      <h3 id="call-modal-name" class="text-lg font-semibold"><?php echo htmlspecialchars($other_user_name); ?></h3>
      <p id="call-modal-status">Llamando...</p>
      <div id="call-timer" style="display: none;">00:00</div>
      <div id="call-modal-buttons" class="mt-4">
        <!-- Botones se insertan dinámicamente -->
      </div>
    </div>
    <!-- Indicador de estado de conexión PeerJS -->
    <div id="peer-connection-indicator"></div>
  </div>
  
  <!-- Modal para Historial de Llamadas -->
  <div id="call-history-modal" style="display: none; position: fixed; top: 0; left: 0; width: 100%; height: 100%; background: rgba(0, 0, 0, 0.7); z-index: 2000; align-items: center; justify-content: center;">
    <div style="background: white; width: 90%; max-width: 500px; max-height: 80vh; border-radius: 12px; overflow: hidden; display: flex; flex-direction: column;">
      <div class="blue-gradient text-white p-3 flex items-center justify-between">
        <h3 class="text-lg font-semibold">Historial de Llamadas</h3>
        <button id="close-history-btn" class="text-white hover:text-blue-200">
          <i class="fas fa-times"></i>
        </button>
      </div>
      <div id="call-history-content" style="flex: 1; overflow-y: auto; padding: 1rem;">
        <div class="text-center py-4 text-gray-500">
          <i class="fas fa-spinner fa-spin mr-2"></i> Cargando historial...
        </div>
      </div>
    </div>
  </div>

  <!-- Scripts JS -->
  <script src="https://cdn.jsdelivr.net/npm/emoji-mart@latest/dist/browser.js"></script>
  <script src="https://cdnjs.cloudflare.com/ajax/libs/photoswipe/5.4.4/umd/photoswipe.umd.min.js"></script>
  <script src="https://cdnjs.cloudflare.com/ajax/libs/photoswipe/5.4.4/umd/photoswipe-lightbox.umd.min.js"></script>

  <script>
    // ============================
    // --- Registro de Llamadas ---
    // ============================
    async function logCallEvent(status, otherUserId, isOutgoing = true, startTime = null, endTime = null, duration = null) {
      try {
        const response = await fetch('log_call.php', {
          method: 'POST',
          headers: {
            'Content-Type': 'application/json',
          },
          body: JSON.stringify({
            status: status,
            other_user_id: otherUserId,
            is_caller: isOutgoing,
            start_time: startTime,
            end_time: endTime,
            duration: duration
          })
        });
        
        if (!response.ok) {
          console.warn('Error al registrar evento de llamada:', response.statusText);
          return null;
        }
        
        const data = await response.json();
        return data;
      } catch (error) {
        console.error('Error al registrar llamada:', error);
        return null;
      }
    }

    // Función para calcular la duración de la llamada en segundos
    function calculateCallDuration(startTime) {
      if (!startTime) return 0;
      const start = new Date(startTime);
      const end = new Date();
      return Math.floor((end - start) / 1000);
    }
    
    // Función para obtener historial de llamadas con un usuario específico
    async function getCallHistory(otherUserId, limit = 10) {
      try {
        const response = await fetch(`get_call_history.php?other_user_id=${otherUserId}&limit=${limit}`);
        if (!response.ok) {
          console.warn('Error al obtener historial de llamadas:', response.statusText);
          return [];
        }
        
        const data = await response.json();
        return data.status === 'success' ? data.calls : [];
      } catch (error) {
        console.error('Error al obtener historial de llamadas:', error);
        return [];
      }
    }
    
    // =========================
    // --- Funciones de Chat ---
    // =========================
    const chatMessagesDiv = document.getElementById('chat-messages');
    const messageInput = document.getElementById('message-input');
    const sendBtn = document.getElementById('send-btn');
    const emojiBtn = document.getElementById('emoji-btn');
    const fileInput = document.getElementById('file-input');
    const typingIndicator = document.getElementById('typing-indicator');
    const statusText = document.getElementById('status-text');
    const statusDotIndicator = document.getElementById('status-dot-indicator');
    const emojiPickerContainer = document.getElementById('emoji-picker-container');
    const currentUserId = <?php echo json_encode($user_id); ?>;
    const otherUserId = <?php echo json_encode($other_user_id); ?>;
    const otherUserName = <?php echo json_encode($other_user_name); ?>;
    let lastMessageId = 0;
    let emojiPicker = null;
    let fetchInterval = null;
    let isFetching = false;
    let optimisticMessageIdCounter = 0;
    let lightboxInstance = null;
    const MAX_FILE_SIZE_MB = 50;
    const MAX_FILE_SIZE_BYTES = MAX_FILE_SIZE_MB * 1024 * 1024;
    
    // Detector de navegador para ajustes específicos
    const isSafari = /^((?!chrome|android).)*safari/i.test(navigator.userAgent);
    const isIOS = /iPad|iPhone|iPod/.test(navigator.userAgent) && !window.MSStream;
    
    if (isSafari || isIOS) {
      console.log("Navegador Safari/iOS detectado, aplicando ajustes específicos");
      // Aplicar cualquier ajuste específico para Safari aquí si es necesario
    }
    
    function renderMessage(msg, isOptimistic = false, tempId = null) {
      if (!msg || (!msg.id && !tempId)) { console.error("renderMessage: Datos inválidos", msg); return; }
      const messageId = msg.id || tempId;
      const existingElement = chatMessagesDiv.querySelector(`[data-message-id="${messageId}"]`);
      if (!isOptimistic && msg.id && tempId) { 
        const el = chatMessagesDiv.querySelector(`[data-message-id="${tempId}"]`);
        if (el) el.remove();
        if (chatMessagesDiv.querySelector(`[data-message-id="${msg.id}"]`)) return;
      } else if (existingElement) { return; }
  
      const messageElement = document.createElement('div');
      const isSender = msg.sender_id === currentUserId;
      messageElement.classList.add('mb-3', 'flex', isSender ? 'justify-end' : 'justify-start');
      messageElement.dataset.messageId = messageId;
  
      let contentHTML = '';
      const escapedMessage = msg.message ? msg.message.replace(/</g, "&lt;").replace(/>/g, "&gt;") : '';
  
      if (isOptimistic && msg.type !== 'text') {
        contentHTML = `<div class="flex items-center p-3 rounded-lg bg-blue-600 text-white opacity-70 max-w-xs md:max-w-sm">
                        <div class="spinner-tiny mr-3 flex-shrink-0"></div>
                        <div class="min-w-0">
                          <div class="font-medium truncate">${msg.file_name || 'Subiendo...'}</div>
                          <div class="text-xs text-blue-200">${formatFileSize(msg.file_size) || ''}</div>
                          ${escapedMessage ? `<p class="mt-1 text-sm text-blue-100 break-words">${escapedMessage}</p>` : ''}
                        </div>
                      </div>`;
      } else {
        switch (msg.type) {
          case 'image': 
            const thumbSrc = msg.file_url || msg.temp_preview_url || '';
            const largeSrc = msg.file_url || '';
            const canOpenLightbox = largeSrc && typeof window.PhotoSwipeLightbox !== 'undefined';
            contentHTML = `<div class="max-w-xs md:max-w-sm ${canOpenLightbox ? 'cursor-pointer' : ''} message-image-container"
                              ${canOpenLightbox ? `onclick="openPhotoSwipe(event, '${largeSrc.replace(/'/g, "\\'")}')"` : ''}>
                              <img src="${thumbSrc}" alt="${msg.file_name || 'Imagen'}" class="rounded-lg max-w-full h-auto shadow-md block">
                              ${escapedMessage ? `<p class="mt-1 text-sm ${isSender ? 'text-white opacity-90' : 'text-gray-700'} break-words">${escapedMessage}</p>` : ''}
                            </div>`;
            break;
          case 'video':
            // Mejora para Safari: asegurarse de que el atributo playsinline esté presente
            contentHTML = `<div class="max-w-xs md:max-w-sm">
                              <video controls playsinline class="rounded-lg max-w-full h-auto shadow-md block" preload="metadata">
                                <source src="${msg.file_url}" type="${msg.file_type || 'video/mp4'}">
                                Video no soportado.
                                <a href="${msg.file_url}" target="_blank" class="underline ${isSender ? 'text-blue-200 hover:text-white' : 'text-blue-600 hover:text-blue-800'}">Descargar</a>
                              </video>
                              ${escapedMessage ? `<p class="mt-1 text-sm ${isSender ? 'text-white opacity-90' : 'text-gray-700'} break-words">${escapedMessage}</p>` : ''}
                            </div>`;
            break;
          case 'file': 
            const fileSizeFormatted = formatFileSize(msg.file_size);
            contentHTML = `<a href="${msg.file_url}" target="_blank" download="${msg.file_name || 'archivo'}" 
                              class="flex items-center p-3 rounded-lg ${isSender ? 'bg-blue-600 text-white' : 'bg-gray-300 text-gray-800'} 
                              hover:bg-opacity-80 transition duration-150 shadow-sm max-w-xs md:max-w-sm">
                              <i class="fas ${getFileIcon(msg.file_type || '')} fa-lg mr-3 flex-shrink-0 ${isSender ? 'text-blue-100' : 'text-gray-600'}"></i>
                              <div class="min-w-0">
                                <div class="font-medium truncate">${msg.file_name || 'Archivo adjunto'}</div>
                                <div class="text-xs ${isSender ? 'text-blue-200' : 'text-gray-500'}">${fileSizeFormatted}</div>
                                ${escapedMessage ? `<p class="mt-1 text-sm ${isSender ? 'text-blue-100' : 'text-gray-600'} break-words">${escapedMessage}</p>` : ''}
                              </div>
                            </a>`;
            break;
          default:
            const urlRegex = /(\b(https?|ftp|file):\/\/[-A-Z0-9+&@#\/%?=~_|!:,.;]*[-A-Z0-9+&@#\/%=~_|])/ig;
            const linkedMessage = escapedMessage.replace(urlRegex, (url) => `<a href="${url}" target="_blank" rel="noopener noreferrer" class="underline ${isSender ? 'text-white hover:text-blue-200' : 'text-blue-600 hover:text-blue-800'}">${url}</a>`);
            contentHTML = `<p class="whitespace-pre-wrap break-words">${linkedMessage}</p>`;
            break;
        }
      }
  
      let messageDate;
      if (msg.timestamp_iso) { 
        messageDate = new Date(msg.timestamp_iso); 
      } else if (msg.timestamp) { 
        messageDate = new Date(msg.timestamp); 
      } else { 
        messageDate = new Date(); 
      }
  
      let timeString = 'Hora inválida';
      if (messageDate instanceof Date && !isNaN(messageDate.getTime())) {
        try {
          const timeFormatter = new Intl.DateTimeFormat('es-ES', { hour: 'numeric', minute: '2-digit', hour12: false });
          timeString = timeFormatter.format(messageDate);
        } catch (e) { 
          console.error("Error formateando fecha:", e, messageDate);
          timeString = 'Error hora';
        }
      }
  
      const statusIconHTML = isSender ? (isOptimistic ? `<i class="fas fa-clock ml-1 message-status-icon" title="Enviando..." style="font-size: 0.7rem; opacity: 0.6;"></i>` : renderMessageStatus(msg)) : '';
      const bubbleClasses = `p-3 rounded-lg shadow-md max-w-lg ${ isSender ? 'bg-blue-500 text-white rounded-br-none' : 'bg-white text-gray-800 rounded-bl-none' }`;
  
      messageElement.innerHTML = `<div class="${bubbleClasses}">
                                    ${contentHTML}
                                    <div class="text-xs mt-1 ${isSender ? 'text-blue-200 opacity-90' : 'text-gray-400'} text-right flex items-center justify-end">
                                      <span>${timeString}</span>${statusIconHTML}
                                    </div>
                                  </div>`;
      chatMessagesDiv.appendChild(messageElement);
    }
  
    function openPhotoSwipe(event, imageUrl) {
      event.preventDefault();
      event.stopPropagation();
      if (!imageUrl || typeof window.PhotoSwipeLightbox === 'undefined') {
        console.error("PhotoSwipe no cargado/URL inválida.");
        alert("Error al abrir imagen.");
        return;
      }
      const img = new Image();
      img.onload = () => initializeAndOpenPhotoSwipe([{ src: imageUrl, w: img.naturalWidth || 1024, h: img.naturalHeight || 768 }]);
      img.onerror = () => {
        console.warn("No se pudo precargar imagen:", imageUrl);
        initializeAndOpenPhotoSwipe([{ src: imageUrl, w: 1024, h: 768 }]);
      };
      img.src = imageUrl;
    }
  
    function initializeAndOpenPhotoSwipe(slideData) {
      if (lightboxInstance) {
        lightboxInstance.destroy();
        lightboxInstance = null;
      }
      lightboxInstance = new PhotoSwipeLightbox({
        dataSource: slideData,
        pswpModule: window.PhotoSwipe
      });
      lightboxInstance.on('error', ({ error, D }) => console.error('PhotoSwipeLightbox error:', error, D));
      lightboxInstance.init();
      lightboxInstance.loadAndOpen(0);
    }
  
    function getFileIcon(mimeType) {
      if (!mimeType) return 'fa-file';
      if (mimeType.startsWith('image/')) return 'fa-file-image';
      if (mimeType.startsWith('video/')) return 'fa-file-video';
      if (mimeType.startsWith('audio/')) return 'fa-file-audio';
      if (mimeType === 'application/pdf') return 'fa-file-pdf';
      if (mimeType.includes('word')) return 'fa-file-word';
      if (mimeType.includes('excel') || mimeType.includes('spreadsheetml')) return 'fa-file-excel';
      if (mimeType.includes('powerpoint') || mimeType.includes('presentationml')) return 'fa-file-powerpoint';
      if (mimeType.includes('zip') || mimeType.includes('rar') || mimeType.includes('compressed')) return 'fa-file-archive';
      if (mimeType.startsWith('text/')) return 'fa-file-alt';
      return 'fa-file';
    }
  
    function renderMessageStatus(msg) {
      const isRead = parseInt(msg.read || 0);
      const isDelivered = parseInt(msg.delivered || 0);
      let iconClass = 'fa-clock';
      let tooltip = 'Enviado';
      if (isDelivered) { iconClass = 'fa-check'; tooltip = 'Entregado'; }
      if (isRead) { iconClass = 'fa-check-double text-blue-400'; tooltip = 'Leído'; }
      return `<i class="fas ${iconClass} ml-1 message-status-icon" title="${tooltip}" style="font-size: 0.75rem; opacity: 0.8;"></i>`;
    }
  
    function formatFileSize(bytes) {
      if (bytes === null || bytes === undefined || isNaN(bytes)) return '';
      if (bytes === 0) return '0 Bytes';
      const k = 1024;
      const sizes = ['Bytes', 'KB', 'MB', 'GB', 'TB'];
      const i = Math.floor(Math.log(bytes) / Math.log(k));
      const size = parseFloat((bytes / Math.pow(k, i)).toFixed(i > 0 ? 1 : 0));
      return size + ' ' + sizes[i];
    }
  
    function scrollToBottom(force = false) {
      const isScrolledToBottom = chatMessagesDiv.scrollHeight - chatMessagesDiv.scrollTop - chatMessagesDiv.clientHeight < 150;
      if (force || isScrolledToBottom) {
        setTimeout(() => {
          chatMessagesDiv.scrollTop = chatMessagesDiv.scrollHeight;
        }, 50);
      }
    }
  
    function loadInitialMessages() {
      let maxId = 0;
      const initialMessages = <?php echo json_encode($messages); ?> || [];
      initialMessages.forEach(msg => {
        renderMessage(msg);
        const msgId = parseInt(msg.id);
        if (msgId > maxId) maxId = msgId;
      });
      lastMessageId = maxId;
      console.log("ID último mensaje inicial:", lastMessageId);
      setTimeout(() => scrollToBottom(true), 300);
    }
  
    function updateStatusUI(status) {
      const isOnline = status === 'online';
      statusText.textContent = isOnline ? 'Online' : 'Offline';
      statusDotIndicator.classList.toggle('bg-green-500', isOnline);
      statusDotIndicator.classList.toggle('bg-gray-500', !isOnline);
    }
  
    async function fetchNewMessages() {
      if (isFetching) return;
      isFetching = true;
      try {
        const url = `get_new_messages.php?other_user_id=${otherUserId}&last_message_id=${lastMessageId}`;
        const response = await fetch(url);
        if (!response.ok) { console.error('Error fetchNewMessages:', response.statusText); isFetching = false; return; }
        const data = await response.json();
        if (data.status === 'success') {
          let newMessagesRendered = false;
          if (data.messages && data.messages.length > 0) {
            data.messages.forEach(msg => {
              if (!chatMessagesDiv.querySelector(`[data-message-id="${msg.id}"]`)) {
                renderMessage(msg);
                newMessagesRendered = true;
              }
            });
          }
          if (data.read_by_other && data.read_by_other.length > 0) {
            data.read_by_other.forEach(readId => {
              const el = chatMessagesDiv.querySelector(`[data-message-id="${readId}"]`);
              if (el) {
                const icon = el.querySelector('.message-status-icon');
                if (icon && !icon.classList.contains('fa-check-double')) {
                  icon.className = 'fas fa-check-double text-blue-400 ml-1 message-status-icon';
                  icon.title = 'Leído';
                  icon.style.cssText = 'font-size: 0.75rem; opacity: 0.8;';
                }
              }
            });
          }
          if (data.last_processed_id && parseInt(data.last_processed_id) > lastMessageId) {
            lastMessageId = parseInt(data.last_processed_id);
          }
          if (newMessagesRendered) { scrollToBottom(); }
          if (data.user_status && data.user_status.status) { updateStatusUI(data.user_status.status); }
          if (typeof data.is_typing !== 'undefined') {
            typingIndicator.classList.toggle('hidden', !data.is_typing);
            if (data.is_typing) typingIndicator.querySelector('#typing-user-name').textContent = otherUserName;
          }
        } else if (data.message === 'No autenticado') {
          clearInterval(fetchInterval);
          alert('Sesión expirada. Inicia sesión de nuevo.');
          window.location.href = 'login.php';
        }
        else if (data.message === 'No new messages') {
          if (data.user_status && data.user_status.status) { updateStatusUI(data.user_status.status); }
          if (typeof data.is_typing !== 'undefined') {
            typingIndicator.classList.toggle('hidden', !data.is_typing);
            if (data.is_typing) typingIndicator.querySelector('#typing-user-name').textContent = otherUserName;
          }
        }
        else { console.warn('Respuesta fetchNewMessages:', data.message || 'Desconocida'); }
      } catch (error) {
        console.error('Error fatal fetchNewMessages:', error);
      } finally {
        isFetching = false;
      }
    }
  
    async function sendMessage() {
      const messageText = messageInput.value.trim();
      const file = fileInput.files[0];
      if (!messageText && !file) return;
      if (file && file.size > MAX_FILE_SIZE_BYTES) {
        alert(`Archivo "${file.name}" (${formatFileSize(file.size)}) excede límite de ${MAX_FILE_SIZE_MB} MB.`);
        fileInput.value = '';
        messageInput.placeholder = "Escribe un mensaje...";
        return;
      }
      optimisticMessageIdCounter++;
      const tempId = `optimistic-${optimisticMessageIdCounter}`;
      const optimisticMessageData = {
        sender_id: currentUserId,
        receiver_id: otherUserId,
        message: messageText,
        type: file ? (file.type.startsWith('image/') ? 'image' : (file.type.startsWith('video/') ? 'video' : 'file')) : 'text',
        timestamp: new Date().toISOString(),
        file_name: file ? file.name : null,
        file_size: file ? file.size : null,
        file_type: file ? file.type : null,
        delivered: 0,
        read: 0
      };
      renderMessage(optimisticMessageData, true, tempId);
      scrollToBottom(true);
      const previousPlaceholder = messageInput.placeholder;
      messageInput.value = '';
      fileInput.value = '';
      messageInput.placeholder = "Escribe un mensaje...";
      const formData = new FormData();
      if (typeof otherUserId !== 'number' || otherUserId <= 0) {
        console.error('Error sendMessage: otherUserId inválido:', otherUserId);
        const el = chatMessagesDiv.querySelector(`[data-message-id="${tempId}"]`);
        if (el) el.remove();
        alert('Error: Destinatario inválido.');
        return;
      }
      formData.append('receiver_id', otherUserId);
      if (messageText) formData.append('message', messageText);
      if (file) formData.append('file', file);
      sendBtn.disabled = true;
      messageInput.disabled = true;
      emojiBtn.disabled = true;
      try {
        const response = await fetch('send_message.php', { method: 'POST', body: formData });
        const data = await response.json();
        const optimisticElement = chatMessagesDiv.querySelector(`[data-message-id="${tempId}"]`);
        if (optimisticElement) optimisticElement.remove();
        if (data.status === 'success' && data.message && data.message.id) {
          renderMessage(data.message, false);
          if (parseInt(data.message.id) > lastMessageId) lastMessageId = parseInt(data.message.id);
        }
        else {
          alert(`Error al enviar: ${data.message || 'Respuesta inválida.'}`);
          console.error("Error send_message.php:", data);
          if (messageText) messageInput.value = messageText;
          if (file) messageInput.placeholder = previousPlaceholder;
        }
      } catch (error) {
        console.error('Error de red al enviar:', error);
        alert('Error de conexión al enviar. Inténtalo de nuevo.');
        const el = chatMessagesDiv.querySelector(`[data-message-id="${tempId}"]`);
        if (el) el.remove();
        if (messageText) messageInput.value = messageText;
        if (file) messageInput.placeholder = previousPlaceholder;
      }
      finally {
        sendBtn.disabled = false;
        messageInput.disabled = false;
        emojiBtn.disabled = false;
        messageInput.focus();
      }
    }
  
    // Renderizar el historial de llamadas
    async function renderCallHistory() {
      const callHistoryModal = document.getElementById('call-history-modal');
      const callHistoryContent = document.getElementById('call-history-content');
      
      // Mostrar modal con indicador de carga
      callHistoryModal.style.display = 'flex';
      callHistoryContent.innerHTML = `
        <div class="text-center py-4 text-gray-500">
          <i class="fas fa-spinner fa-spin mr-2"></i> Cargando historial...
        </div>
      `;
      
      try {
        // Obtener historial
        const calls = await getCallHistory(otherUserId, 50);
        
        if (calls.length === 0) {
          callHistoryContent.innerHTML = `
            <div class="text-center py-4 text-gray-500">
              No hay registros de llamadas con este contacto.
            </div>
          `;
          return;
        }
        
        // Preparar HTML para mostrar cada llamada
        let html = `<div class="space-y-3">`;
        
        calls.forEach(call => {
          let iconClass = 'fa-phone-alt';
          let colorClass = 'text-blue-500';
          let statusText = '';
          let durationText = '';
          
          // Determinar ícono y estado basado en el estado y si es saliente o entrante
          if (call.is_outgoing) {
            if (call.status === 'missed' || call.status === 'rejected') {
              iconClass = 'fa-phone-slash';
              colorClass = 'text-red-500';
              statusText = 'Llamada sin respuesta';
            } else if (call.status === 'initiated') {
              iconClass = 'fa-phone-volume';
              colorClass = 'text-yellow-500';
              statusText = 'Llamando...';
            } else if (call.status === 'answered') {
              iconClass = 'fa-phone';
              colorClass = 'text-green-500';
              statusText = 'Llamada saliente';
            } else if (call.status === 'ended') {
              iconClass = 'fa-phone';
              colorClass = 'text-green-500';
              statusText = 'Llamada saliente';
              durationText = call.duration_formatted;
            }
          } else {
            // Llamada entrante
            if (call.status === 'missed') {
              iconClass = 'fa-phone-slash';
              colorClass = 'text-red-500';
              statusText = 'Llamada perdida';
            } else if (call.status === 'rejected') {
              iconClass = 'fa-phone-slash';
              colorClass = 'text-red-500';
              statusText = 'Llamada rechazada';
            } else if (call.status === 'answered' || call.status === 'ended') {
              iconClass = 'fa-phone';
              colorClass = 'text-green-500';
              statusText = 'Llamada entrante';
              if (call.status === 'ended') {
                durationText = call.duration_formatted;
              }
            }
          }
          
          html += `
            <div class="flex items-center p-3 bg-gray-50 rounded-lg">
              <div class="mr-3 h-10 w-10 rounded-full flex items-center justify-center ${colorClass} bg-gray-200">
                <i class="fas ${iconClass}"></i>
              </div>
              <div class="flex-1">
                <div class="flex items-center justify-between">
                  <span class="font-medium">${statusText}</span>
                  <span class="text-sm text-gray-500">${call.date_formatted}</span>
                </div>
                ${durationText ? `<div class="text-sm text-gray-600">Duración: ${durationText}</div>` : ''}
              </div>
              <div>
                <button 
                  class="ml-2 p-2 text-blue-500 hover:text-blue-700 call-again-btn" 
                  data-call-id="${call.id}" 
                  title="Llamar de nuevo">
                  <i class="fas fa-phone"></i>
                </button>
              </div>
            </div>
          `;
        });
        
        html += `</div>`;
        callHistoryContent.innerHTML = html;
        
        // Agregar eventos a los botones de llamar de nuevo
        document.querySelectorAll('.call-again-btn').forEach(btn => {
          btn.addEventListener('click', () => {
            // Cerrar el modal
            callHistoryModal.style.display = 'none';
            // Iniciar una nueva llamada
            initiateCall();
          });
        });
        
      } catch (err) {
        console.error("Error al obtener historial de llamadas:", err);
        callHistoryContent.innerHTML = `
          <div class="text-center py-4 text-red-500">
            <i class="fas fa-exclamation-circle mr-2"></i> 
            Error al cargar el historial de llamadas.
          </div>
        `;
      }
    }
    
    document.addEventListener('DOMContentLoaded', () => {
      if (typeof window.EmojiMart === 'undefined') { console.error("EmojiMart no cargado!"); emojiBtn.style.display = 'none'; }
      if (typeof window.PhotoSwipeLightbox === 'undefined' || typeof window.PhotoSwipe === 'undefined') { console.error("PhotoSwipe no cargado!"); }
      if (typeof otherUserId !== 'number' || otherUserId <= 0) {
        console.error("FATAL: otherUserId inválido:", otherUserId);
        alert("Error cargando chat.");
        messageInput.disabled = true;
        sendBtn.disabled = true;
        emojiBtn.disabled = true;
        messageInput.placeholder = "Error";
        return;
      }
      loadInitialMessages();
      fetchInterval = setInterval(fetchNewMessages, 3000);
      setTimeout(() => { messageInput.focus(); }, 150);
      sendBtn.addEventListener('click', sendMessage);
      messageInput.addEventListener('keypress', (e) => {
        if (e.key === 'Enter' && !e.shiftKey) {
          e.preventDefault();
          sendMessage();
        }
      });
      
      // Configurar evento para botón de historial de llamadas
      const callHistoryBtn = document.getElementById('call-history-btn');
      const closeHistoryBtn = document.getElementById('close-history-btn');
      const callHistoryModal = document.getElementById('call-history-modal');
      
      callHistoryBtn.addEventListener('click', () => {
        renderCallHistory();
      });
      
      closeHistoryBtn.addEventListener('click', () => {
        callHistoryModal.style.display = 'none';
      });
      
      // Cerrar modal con clic fuera del contenido
      callHistoryModal.addEventListener('click', (e) => {
        if (e.target === callHistoryModal) {
          callHistoryModal.style.display = 'none';
        }
      });
      
      // Cerrar modal con ESC
      document.addEventListener('keydown', (e) => {
        if (e.key === 'Escape' && callHistoryModal.style.display === 'flex') {
          callHistoryModal.style.display = 'none';
        }
      });
      emojiBtn.addEventListener('click', (event) => {
        if (typeof window.EmojiMart === 'undefined' || typeof window.EmojiMart.Picker === 'undefined') {
          console.error("EmojiMart no listo.");
          return;
        }
        event.stopPropagation();
        if (emojiPickerContainer.style.display === 'block') {
          emojiPickerContainer.style.display = 'none';
        } else {
          if (!emojiPicker) {
            try {
              emojiPicker = new EmojiMart.Picker({
                onEmojiSelect: (emoji) => {
                  const start = messageInput.selectionStart;
                  const end = messageInput.selectionEnd;
                  messageInput.value = messageInput.value.substring(0, start) + emoji.native + messageInput.value.substring(end);
                  messageInput.selectionStart = messageInput.selectionEnd = start + emoji.native.length;
                  messageInput.focus();
                  if (emojiPickerContainer) emojiPickerContainer.style.display = 'none';
                },
                locale: 'es',
                theme: 'light',
                set: 'native',
                previewPosition: 'none',
                searchPosition: 'none',
                navPosition: 'bottom',
                perLine: 8
              });
              emojiPickerContainer.innerHTML = '';
              emojiPickerContainer.appendChild(emojiPicker);
            } catch (e) {
              console.error("Error EmojiMart Picker:", e);
              alert("Error al cargar emojis.");
              return;
            }
          }
          emojiPickerContainer.style.display = 'block';
        }
      });
      document.addEventListener('click', (event) => {
        if (emojiPickerContainer.style.display === 'block' && !emojiPickerContainer.contains(event.target) && event.target !== emojiBtn && !emojiBtn.contains(event.target)) {
          emojiPickerContainer.style.display = 'none';
        }
      });
      fileInput.addEventListener('change', (event) => {
        const file = event.target.files[0];
        if (file) {
          if (file.size > MAX_FILE_SIZE_BYTES) {
            alert(`Archivo muy grande (${formatFileSize(file.size)}). Límite ${MAX_FILE_SIZE_MB} MB.`);
            fileInput.value = '';
            messageInput.placeholder = "Escribe un mensaje...";
          } else {
            messageInput.placeholder = `Archivo: ${file.name}`;
          }
        } else {
          messageInput.placeholder = "Escribe un mensaje...";
        }
      });
      document.addEventListener('keydown', (e) => {
        if (e.key === 'Escape' && window.lightboxInstance && window.lightboxInstance.pswp && window.lightboxInstance.pswp.isOpen) {
          window.lightboxInstance.close();
        }
      });
    });
  
    // ================================================================
    // --- Sistema de Llamadas Mejorado con Soporte para Safari/iOS ---
    // ================================================================
    
    // Elementos del UI
    const callModal = document.getElementById('call-modal');
    const callModalContent = document.getElementById('call-modal-content');
    const callModalName = document.getElementById('call-modal-name');
    const callModalStatus = document.getElementById('call-modal-status');
    const callModalButtons = document.getElementById('call-modal-buttons');
    const callTimer = document.getElementById('call-timer');
    const audioCallBtn = document.getElementById('audio-call-btn');
    const peerConnectionIndicator = document.getElementById('peer-connection-indicator');
    
    // Variables para el sistema de llamadas
    let peer = null;
    let localCallStream = null;
    let currentCall = null;
    let callTimerInterval = null;
    let callStartTime = null;
    let isMuted = false;
    let callHeartbeatInterval = null;
    let callMonitorInterval = null;
    let reconnectAttempts = 0;
    let heartbeatFailCount = 0;
    const MAX_RECONNECT_ATTEMPTS = 3;
    let peerOptions = {
      debug: 1, // 0 = desactivado, 1 = solo errores, 2 = advertencias, 3 = todo
      config: {
        iceServers: [
          { urls: 'stun:stun.l.google.com:19302' },
          { urls: 'stun:stun1.l.google.com:19302' },
          { urls: 'stun:stun2.l.google.com:19302' }
        ],
        // Safari requiere estas configuraciones específicas
        sdpSemantics: 'unified-plan',
        iceTransportPolicy: 'all',
      }
    };
    
    // Función para mostrar el estado de PeerJS
    function updatePeerStatus(status, error = null) {
      if (!peerConnectionIndicator) return;
      
      let statusText = "";
      let statusColor = "";
      
      switch (status) {
        case 'disconnected':
          statusText = "Desconectado";
          statusColor = "#e53e3e"; // rojo
          break;
        case 'connected':
          statusText = "Conectado";
          statusColor = "#48bb78"; // verde
          break;
        case 'connecting':
          statusText = "Conectando...";
          statusColor = "#ecc94b"; // amarillo
          break;
        case 'error':
          statusText = `Error: ${error || 'desconocido'}`;
          statusColor = "#e53e3e"; // rojo
          break;
        default:
          statusText = status;
          statusColor = "#a0aec0"; // gris
      }
      
      peerConnectionIndicator.textContent = statusText;
      peerConnectionIndicator.style.color = statusColor;
    }
    
    // Mostrar modal de llamada con mensaje y botones personalizados
    function showCallModal(message, buttonsHTML) {
      callModalStatus.textContent = message;
      callModalButtons.innerHTML = buttonsHTML;
      callModal.style.display = 'flex';
    }
    
    // Ocultar modal de llamada
    function hideCallModal() {
      callModal.style.display = 'none';
      stopCallTimer();
    }
    
    // Iniciar el cronómetro de llamada
    function startCallTimer() {
      callStartTime = new Date();
      callTimer.style.display = 'inline-block';
      updateCallTimer();
      callTimerInterval = setInterval(updateCallTimer, 1000);
    }
    
    // Detener el cronómetro de llamada
    function stopCallTimer() {
      if (callTimerInterval) {
        clearInterval(callTimerInterval);
        callTimerInterval = null;
      }
      callTimer.style.display = 'none';
    }
    
    // Actualizar el cronómetro de llamada
    function updateCallTimer() {
      if (!callStartTime) return;
      
      const now = new Date();
      const diffInSeconds = Math.floor((now - callStartTime) / 1000);
      const minutes = Math.floor(diffInSeconds / 60).toString().padStart(2, '0');
      const seconds = (diffInSeconds % 60).toString().padStart(2, '0');
      
      callTimer.textContent = `${minutes}:${seconds}`;
    }
    
    // Iniciar un heartbeat para mantener la conexión activa
    function startCallHeartbeat(call) {
      if (callHeartbeatInterval) clearInterval(callHeartbeatInterval);
      
      // Cada 3 segundos enviamos datos pequeños para mantener la conexión activa
      // y confirmar que el otro lado sigue conectado
      callHeartbeatInterval = setInterval(() => {
        try {
          // Enviar por el canal de datos establecido
          let messageSent = false;
          
          if (call.dataChannel && call.dataChannel.readyState === 'open') {
            call.dataChannel.send(JSON.stringify({ 
              type: 'heartbeat', 
              timestamp: Date.now(),
              callActive: true
            }));
            messageSent = true;
          } 
          else if (call.incomingChannel && call.incomingChannel.readyState === 'open') {
            call.incomingChannel.send(JSON.stringify({ 
              type: 'heartbeat', 
              timestamp: Date.now(),
              callActive: true  
            }));
            messageSent = true;
          }
          
          // Si no se pudo enviar el heartbeat, intentar crear un nuevo canal
          if (!messageSent) {
            // Comprobar el estado de la conexión
            const iceState = call.peerConnection.iceConnectionState;
            const connState = call.peerConnection.connectionState;
            
            if ((iceState === 'disconnected' || iceState === 'failed' || iceState === 'closed') ||
                (connState === 'disconnected' || connState === 'failed' || connState === 'closed')) {
              console.warn(`Heartbeat falló. Estado de la conexión: ${connState}, ICE: ${iceState}`);
              heartbeatFailCount = (heartbeatFailCount || 0) + 1;
              
              // Si fallan varios heartbeats consecutivos, terminar llamada
              if (heartbeatFailCount >= 3) {
                console.error("Múltiples fallos de heartbeat. Finalizando llamada.");
                endCurrentCall("Se perdió la conexión");
              }
            }
          } else {
            heartbeatFailCount = 0; // Resetear contador si el mensaje se envía
          }
        } catch (err) {
          console.warn('Error en heartbeat:', err);
          heartbeatFailCount = (heartbeatFailCount || 0) + 1;
          
          if (heartbeatFailCount >= 3) {
            endCurrentCall("Error de comunicación");
          }
        }
      }, 3000);
    }
    
    // Detener el heartbeat
    function stopCallHeartbeat() {
      if (callHeartbeatInterval) {
        clearInterval(callHeartbeatInterval);
        callHeartbeatInterval = null;
      }
    }
    
    // Iniciar el monitoreo de calidad de la llamada
    function startCallMonitoring(call) {
      if (callMonitorInterval) clearInterval(callMonitorInterval);
      
      // Inicializar contadores de monitoreo
      let consecutiveIssuesCount = 0;
      
      callMonitorInterval = setInterval(() => {
        if (!call || !call.peerConnection) {
          clearInterval(callMonitorInterval);
          callMonitorInterval = null;
          return;
        }
        
        try {
          // Verificar el estado de la conexión
          const connectionState = call.peerConnection.connectionState;
          const iceConnectionState = call.peerConnection.iceConnectionState;
          const signalingState = call.peerConnection.signalingState;
          
          console.log(`Monitor de llamada - conexión: ${connectionState}, ICE: ${iceConnectionState}, señalización: ${signalingState}`);
          
          // Si la conexión está fallando, intentamos reconectar
          if (connectionState === 'failed' || connectionState === 'disconnected' ||
              iceConnectionState === 'failed' || iceConnectionState === 'disconnected' ||
              signalingState === 'closed') {
            
            console.warn(`Conexión inestable detectada: ${connectionState}/${iceConnectionState}/${signalingState}`);
            consecutiveIssuesCount++;
            
            // Intentar reconectar la llamada
            if (consecutiveIssuesCount >= 2 && reconnectAttempts < MAX_RECONNECT_ATTEMPTS) {
              reconnectAttempts++;
              console.log(`Intento de reconexión ${reconnectAttempts}/${MAX_RECONNECT_ATTEMPTS}`);
              
              // Intentar restablecer la conexión ICE
              try {
                if (call.peerConnection.restartIce) {
                  call.peerConnection.restartIce();
                  console.log("Reiniciando conexión ICE");
                }
              } catch (e) {
                console.warn("Error al reiniciar ICE:", e);
              }
              
              // Si la conexión está completamente "failed", terminar la llamada
              if (connectionState === 'failed' || iceConnectionState === 'failed') {
                console.error("Estado de conexión crítico, terminando llamada");
                endCurrentCall("Conexión perdida");
                clearInterval(callMonitorInterval);
                callMonitorInterval = null;
              }
            } 
            // Si hemos superado los intentos, terminar la llamada
            else if (consecutiveIssuesCount >= 4 || reconnectAttempts >= MAX_RECONNECT_ATTEMPTS) {
              console.error("Máximo de intentos de reconexión alcanzado o muchos problemas consecutivos");
              endCurrentCall("No se pudo restablecer la conexión");
              clearInterval(callMonitorInterval);
              callMonitorInterval = null;
            }
          } else {
            // Si la conexión se restablece, reiniciar contador
            if (connectionState === 'connected' && iceConnectionState === 'connected') {
              consecutiveIssuesCount = 0;
              reconnectAttempts = 0;
            }
          }
          
          // Verificar el estado de los tracks de audio remoto
          if (call.remoteStream) {
            const remoteTracks = call.remoteStream.getAudioTracks();
            let allTracksEnded = true;
            
            for (const track of remoteTracks) {
              if (track.readyState === 'live') {
                allTracksEnded = false;
                break;
              }
            }
            
            if (remoteTracks.length > 0 && allTracksEnded) {
              console.warn("Todos los tracks de audio remoto han terminado");
              consecutiveIssuesCount += 2; // Incrementar más rápido si no hay audio
            }
          }
          
          // También verificar el elemento de audio
          const remoteAudio = document.getElementById('remote-audio');
          if (remoteAudio && remoteAudio.srcObject) {
            const audioTracks = remoteAudio.srcObject.getAudioTracks();
            if (audioTracks.length === 0 || audioTracks.every(t => t.readyState !== 'live')) {
              console.warn("No hay tracks de audio activos en el elemento de audio");
              consecutiveIssuesCount++;
            }
          }
        } catch (err) {
          console.error("Error en el monitoreo de llamada:", err);
          consecutiveIssuesCount++;
          
          if (consecutiveIssuesCount >= 4) {
            endCurrentCall("Error en la supervisión de la llamada");
            clearInterval(callMonitorInterval);
            callMonitorInterval = null;
          }
        }
      }, 2000); // Revisar cada 2 segundos
    }
    
    // Detener el monitoreo de la llamada
    function stopCallMonitoring() {
      if (callMonitorInterval) {
        clearInterval(callMonitorInterval);
        callMonitorInterval = null;
      }
    }
    
    // Función para finalizar la llamada actual con un mensaje opcional
    function endCurrentCall(reason = null) {
      if (!currentCall) return; // Evitar procesamiento múltiple
      
      console.log("Finalizando llamada:", reason);
      
      // Si la llamada estaba activa y tenemos un cronómetro funcionando, registrar duración
      if (callTimerInterval && callStartTime) {
        const duration = calculateCallDuration(callStartTime);
        // El registro se hace en los event handlers específicos (botones de colgar)
        // Esto es por si la llamada termina por razones técnicas
        if (reason && (reason.includes("Error") || reason.includes("perdió") || 
                       reason.includes("desconectado") || reason.includes("terminada"))) {
          logCallEvent('ended', otherUserId, currentCall.peer === `user_${otherUserId}`, 
                       callStartTime.toISOString(), new Date().toISOString(), duration);
        }
      }
      
      try {
        // Notificar al otro usuario antes de colgar (si es posible)
        let notificationSent = false;
        
        // Intentar múltiples métodos para asegurar que la notificación llegue
        // 1. Vía canal de datos específico
        if (currentCall.dataChannel && currentCall.dataChannel.readyState === 'open') {
          currentCall.dataChannel.send(JSON.stringify({ 
            type: 'hangup', 
            reason,
            callStartTime: callStartTime ? callStartTime.toISOString() : null,
            duration: callTimerInterval && callStartTime ? calculateCallDuration(callStartTime) : 0
          }));
          notificationSent = true;
        }
        
        // 2. Vía canal de datos entrante
        if (!notificationSent && currentCall.incomingChannel && 
            currentCall.incomingChannel.readyState === 'open') {
          currentCall.incomingChannel.send(JSON.stringify({ 
            type: 'hangup', 
            reason,
            callStartTime: callStartTime ? callStartTime.toISOString() : null,
            duration: callTimerInterval && callStartTime ? calculateCallDuration(callStartTime) : 0
          }));
          notificationSent = true;
        }
        
        // 3. Intentar con todos los canales de datos disponibles
        if (!notificationSent && currentCall.peerConnection) {
          try {
            const dataSent = sendToAllDataChannels(currentCall.peerConnection, {
              type: 'hangup', 
              reason,
              callStartTime: callStartTime ? callStartTime.toISOString() : null,
              duration: callTimerInterval && callStartTime ? calculateCallDuration(callStartTime) : 0
            });
            
            if (dataSent) notificationSent = true;
          } catch (e) {
            console.warn("Error al enviar a todos los canales:", e);
          }
        }
        
        if (!notificationSent) {
          console.warn("No se pudo enviar notificación de fin de llamada por ningún canal");
        }
      } catch (e) {
        console.warn("Error general al enviar notificación de fin de llamada:", e);
      }
      
      // Guardar referencia local para limpieza
      const callToClose = currentCall;
      currentCall = null; // Marcamos como null primero para evitar recursión
      
      // Cerrar streams de audio locales
      if (localCallStream) {
        try {
          localCallStream.getTracks().forEach(track => {
            track.stop();
          });
        } catch (e) {
          console.warn("Error al detener tracks locales:", e);
        }
      }
      
      // Cerrar la llamada
      try {
        if (callToClose.peerConnection) {
          // Cerrar explícitamente todos los canales de datos
          try {
            closeAllDataChannels(callToClose.peerConnection);
          } catch (e) {
            console.warn("Error al cerrar canales de datos:", e);
          }
          
          // Cerrar la conexión peer
          callToClose.peerConnection.close();
        }
        
        callToClose.close();
      } catch (e) {
        console.warn("Error al cerrar la llamada:", e);
      }
      
      // Detener todos los temporizadores y limpiar
      stopCallTimer();
      stopCallHeartbeat();
      stopCallMonitoring();
      hideCallModal();
      
      // Limpiar elementos de audio
      const remoteAudio = document.getElementById('remote-audio');
      if (remoteAudio) {
        try {
          const remoteStream = remoteAudio.srcObject;
          if (remoteStream) {
            remoteStream.getTracks().forEach(track => {
              track.stop();
            });
          }
          remoteAudio.srcObject = null;
          remoteAudio.remove();
        } catch (e) {
          console.warn("Error al limpiar audio remoto:", e);
        }
      }
      
      reconnectAttempts = 0;
      isMuted = false;
      callStartTime = null;
      heartbeatFailCount = 0;
      
      console.log("Llamada finalizada correctamente");
    }
    
    // Función auxiliar para enviar a todos los canales de datos disponibles
    function sendToAllDataChannels(peerConnection, message) {
      if (!peerConnection) return false;
      
      let messageSent = false;
      const jsonMessage = JSON.stringify(message);
      
      try {
        // Intentar obtener canales de datos
        if (typeof peerConnection.getDataChannels === 'function') {
          const channels = peerConnection.getDataChannels();
          if (channels && channels.length > 0) {
            for (const channel of channels) {
              if (channel.readyState === 'open') {
                channel.send(jsonMessage);
                messageSent = true;
              }
            }
          }
        }
        
        // Si no hay función getDataChannels, buscar en el objeto peerConnection
        if (!messageSent) {
          for (const prop in peerConnection) {
            if (peerConnection[prop] && 
                typeof peerConnection[prop] === 'object' && 
                peerConnection[prop].readyState === 'open' &&
                typeof peerConnection[prop].send === 'function') {
              try {
                peerConnection[prop].send(jsonMessage);
                messageSent = true;
              } catch (e) {
                console.warn(`Error al enviar al canal ${prop}:`, e);
              }
            }
          }
        }
      } catch (e) {
        console.warn("Error general al enviar mensaje a canales:", e);
      }
      
      return messageSent;
    }
    
    // Función auxiliar para cerrar todos los canales de datos
    function closeAllDataChannels(peerConnection) {
      if (!peerConnection) return;
      
      try {
        // Intentar obtener canales de datos
        if (typeof peerConnection.getDataChannels === 'function') {
          const channels = peerConnection.getDataChannels();
          if (channels && channels.length > 0) {
            for (const channel of channels) {
              if (channel.readyState !== 'closed') {
                channel.close();
              }
            }
          }
        }
        
        // Cerrar canales directamente referenciados
        for (const prop in peerConnection) {
          if (peerConnection[prop] && 
              typeof peerConnection[prop] === 'object' && 
              peerConnection[prop].readyState !== 'closed' &&
              typeof peerConnection[prop].close === 'function') {
            try {
              peerConnection[prop].close();
            } catch (e) {
              console.warn(`Error al cerrar el canal ${prop}:`, e);
            }
          }
        }
      } catch (e) {
        console.warn("Error general al cerrar canales de datos:", e);
      }
    }
    
    // Alternar el estado de silencio del micrófono
    function toggleMute() {
      if (!localCallStream) return;
      
      const audioTracks = localCallStream.getAudioTracks();
      if (!audioTracks || !audioTracks.length) return;
      
      isMuted = !isMuted;
      
      for (const track of audioTracks) {
        track.enabled = !isMuted;
      }
      
      // Actualizar el botón de silencio
      const muteBtn = document.getElementById('mute-btn');
      if (muteBtn) {
        if (isMuted) {
          muteBtn.innerHTML = '<i class="fas fa-microphone-slash"></i>';
          muteBtn.classList.add('active');
          muteBtn.title = "Activar micrófono";
        } else {
          muteBtn.innerHTML = '<i class="fas fa-microphone"></i>';
          muteBtn.classList.remove('active');
          muteBtn.title = "Silenciar micrófono";
        }
      }
    }
    
    // Configurar los handlers de eventos para una llamada
    function setupCallEventHandlers(call) {
      if (!call) return;
      
      // Configurar canal de datos para mensajes
      try {
        // Intentar crear un canal de datos si no existe
        const dataChannel = call.peerConnection.createDataChannel('callControl', {
          ordered: true, // Garantiza que los mensajes lleguen en orden
          reliable: true // Garantiza que los mensajes lleguen (retransmisiones automáticas)
        });
        
        dataChannel.onmessage = (event) => {
          handlePeerDataMessage(event.data);
        };
        
        dataChannel.onopen = () => {
          console.log("Canal de datos abierto para control de llamada");
        };
        
        dataChannel.onclose = () => {
          console.log("Canal de datos cerrado");
        };
        
        dataChannel.onerror = (err) => {
          console.error("Error en canal de datos:", err);
        };
        
        call.dataChannel = dataChannel;
        
        // También escuchar por canales de datos entrantes
        call.peerConnection.ondatachannel = (event) => {
          const incomingChannel = event.channel;
          
          incomingChannel.onmessage = (msgEvent) => {
            handlePeerDataMessage(msgEvent.data);
          };
          
          call.incomingChannel = incomingChannel;
        };
      } catch (err) {
        console.warn("Error al configurar canal de datos:", err);
      }
      
      // Monitorear estado de la conexión ICE
      call.peerConnection.oniceconnectionstatechange = () => {
        const iceState = call.peerConnection.iceConnectionState;
        console.log("ICE connection state change:", iceState);
        
        // Si la conexión ICE se desconecta o falla completamente, terminar la llamada
        if (iceState === 'disconnected' || iceState === 'failed' || iceState === 'closed') {
          console.warn(`La conexión ICE cambió a estado ${iceState}`);
          
          // Dar un breve tiempo antes de terminar para permitir reconexiones en caso de 'disconnected'
          if (iceState === 'disconnected') {
            setTimeout(() => {
              const currentState = call.peerConnection.iceConnectionState;
              if (currentState === 'disconnected' || currentState === 'failed' || currentState === 'closed') {
                endCurrentCall(`Conexión perdida (${currentState})`);
              }
            }, 5000); // 5 segundos para intentar reconectar
          } else {
            endCurrentCall(`Conexión terminada (${iceState})`);
          }
        }
      };
      
      // Monitorear estado de la conexión general
      call.peerConnection.onconnectionstatechange = () => {
        const connState = call.peerConnection.connectionState;
        console.log("Connection state change:", connState);
        
        // Si la conexión se pierde completamente, terminar la llamada
        if (connState === 'disconnected' || connState === 'failed' || connState === 'closed') {
          console.warn(`La conexión cambió a estado ${connState}`);
          endCurrentCall(`Conexión terminada (${connState})`);
        }
      };
      
      // Cuando se recibe el stream remoto
      call.on('stream', remoteStream => {
        // Crear o reutilizar el elemento de audio
        let audioElement = document.getElementById('remote-audio');
        if (!audioElement) {
          audioElement = document.createElement('audio');
          audioElement.id = 'remote-audio';
          audioElement.autoplay = true;
          audioElement.muted = false;
          document.body.appendChild(audioElement);
        }
        
        audioElement.srcObject = remoteStream;
        
        // Detectar cuando los tracks del stream remoto terminan (el otro usuario colgó)
        remoteStream.getTracks().forEach(track => {
          track.onended = () => {
            console.log("Track remoto terminado. Posible desconexión.");
            // Esperar brevemente antes de terminar para evitar falsos positivos
            setTimeout(() => {
              // Verificar si todos los tracks se han cerrado
              const allEnded = remoteStream.getTracks().every(t => t.readyState === 'ended');
              if (allEnded && currentCall) {
                endCurrentCall("El otro usuario ha colgado");
              }
            }, 1000);
          };
        });
        
        // En Safari/iOS puede ser necesario iniciar la reproducción manualmente
        try {
          const playPromise = audioElement.play();
          if (playPromise !== undefined) {
            playPromise.catch(error => {
              console.error("Error al reproducir audio remoto:", error);
              // Si falla por interacción del usuario, mostramos un botón para reproducir
              if (error.name === 'NotAllowedError' || error.name === 'NotSupportedError') {
                callModalStatus.innerHTML = 'Toca el botón para activar el audio';
                callModalButtons.innerHTML += `<button id="enable-audio-btn" class="btn-accept"><i class="fas fa-volume-up"></i></button>`;
                
                document.getElementById('enable-audio-btn').addEventListener('click', () => {
                  audioElement.play().catch(e => console.error("Error secundario al reproducir audio:", e));
                  callModalStatus.textContent = 'En llamada';
                  document.getElementById('enable-audio-btn').remove();
                });
              }
            });
          }
        } catch (e) {
          console.error("Error general de audio:", e);
        }
        
        // Actualizar UI para mostrar llamada conectada
        callModalStatus.textContent = 'En llamada';
        
        // Iniciar cronómetro de llamada
        startCallTimer();
        
        // Iniciar heartbeat y monitoreo
        startCallHeartbeat(call);
        startCallMonitoring(call);
      });
      
      // Cuando se cierra la llamada
      call.on('close', () => {
        console.log("Evento close detectado en la llamada");
        endCurrentCall("Llamada finalizada");
      });
      
      // Manejo de errores
      call.on('error', err => {
        console.error("Error en la llamada:", err);
        endCurrentCall(`Error: ${err.message || 'desconocido'}`);
      });
    }
    
    // Manejar mensajes recibidos por el canal de datos
    function handlePeerDataMessage(data) {
      try {
        const message = typeof data === 'string' ? JSON.parse(data) : data;
        
        if (message.type === 'hangup') {
          console.log("Recibido mensaje de colgar llamada:", message);
          
          // Si el otro usuario colgó, registrar el evento
          if (message.callStartTime) {
            const duration = message.duration || calculateCallDuration(new Date(message.callStartTime));
            logCallEvent('ended', otherUserId, false, message.callStartTime, new Date().toISOString(), duration);
          }
          
          // Terminar la llamada localmente
          endCurrentCall(message.reason || "Llamada finalizada por el otro usuario");
        }
        else if (message.type === 'heartbeat') {
          // Confirmar que la conexión sigue activa
          console.debug("Heartbeat recibido:", new Date(message.timestamp).toLocaleTimeString());
        }
        
      } catch (error) {
        console.warn("Error al procesar mensaje de datos:", error, data);
      }
    }
    
    // Inicializar PeerJS con opciones mejoradas
    function initializePeer() {
      if (peer) {
        try {
          peer.destroy();
        } catch(e) {
          console.warn("Error al destruir peer anterior:", e);
        }
      }
      
      // Crear nuevo peer con ID basado en el usuario
      try {
        updatePeerStatus('connecting');
        peer = new Peer(`user_${currentUserId}`, peerOptions);
        
        // Eventos del peer
        peer.on('open', id => {
          console.log("PeerJS conectado con ID:", id);
          updatePeerStatus('connected');
          
          // Una vez conectado, intentamos obtener acceso al audio
          initializeAudio();
        });
        
        peer.on('disconnected', () => {
          console.warn("PeerJS desconectado. Intentando reconectar...");
          updatePeerStatus('disconnected');
          
          // Intentar reconexión
          setTimeout(() => {
            if (peer) peer.reconnect();
          }, 2000);
        });
        
        peer.on('close', () => {
          console.log("Conexión PeerJS cerrada");
          updatePeerStatus('closed');
          
          // Si hay una llamada activa, la terminamos
          if (currentCall) {
            endCurrentCall("Conexión perdida");
          }
        });
        
        peer.on('error', err => {
          console.error("Error PeerJS:", err);
          updatePeerStatus('error', err.type);
          
          // Ciertos errores pueden requerir reinicializar todo
          if (err.type === 'network' || err.type === 'peer-unavailable') {
            if (currentCall) {
              endCurrentCall("Error de conexión");
            }
            
            // Intentar reinicializar después de un tiempo
            setTimeout(initializePeer, 5000);
          }
        });
        
        // Manejo de llamadas entrantes
        peer.on('call', call => {
          handleIncomingCall(call);
        });
        
      } catch(err) {
        console.error("Error al inicializar PeerJS:", err);
        updatePeerStatus('error', 'Inicialización fallida');
        
        // Intentar de nuevo después de un tiempo
        setTimeout(initializePeer, 5000);
      }
    }
    
    // Inicializar el acceso al audio
    function initializeAudio() {
      const constraints = { 
        audio: {
          echoCancellation: true,
          noiseSuppression: true,
          autoGainControl: true
        }
      };
      
      navigator.mediaDevices.getUserMedia(constraints)
        .then(stream => {
          localCallStream = stream;
          console.log("Acceso al audio obtenido");
        })
        .catch(err => {
          console.error("Error al obtener acceso al audio:", err);
          alert("No se pudo acceder al micrófono. Verifica los permisos.");
        });
    }
    
    // Manejar una llamada entrante
    function handleIncomingCall(call) {
      // Solo aceptar llamadas del usuario actual del chat
      if (call.peer !== `user_${otherUserId}`) {
        console.warn(`Rechazando llamada de usuario desconocido: ${call.peer}`);
        call.close();
        return;
      }
      
      // Si ya hay una llamada activa, rechazar la nueva
      if (currentCall) {
        console.warn("Rechazando llamada entrante porque ya hay una activa");
        call.close();
        return;
      }
      
      // Establecer tiempo de inicio para el registro
      callStartTime = new Date();
      
      // Mostrar modal de llamada entrante
      showCallModal(`Llamada entrante de ${otherUserName}`, `
        <button class="btn-accept" id="accept-call-btn"><i class="fas fa-phone"></i></button>
        <button class="btn-decline" id="decline-call-btn"><i class="fas fa-phone-slash"></i></button>
      `);
      
      // Evento para aceptar la llamada
      document.getElementById('accept-call-btn').addEventListener('click', () => {
        // Verificar que tenemos acceso al audio
        if (!localCallStream) {
          console.error("No hay acceso al audio para contestar la llamada");
          alert("No se puede contestar sin acceso al micrófono");
          call.close();
          hideCallModal();
          return;
        }
        
        // Registrar llamada contestada
        logCallEvent('answered', otherUserId, false, callStartTime.toISOString());
        
        // Responder la llamada con nuestro stream
        call.answer(localCallStream);
        currentCall = call;
        setupCallEventHandlers(call);
        
        // Actualizar UI de llamada activa
        showCallModal('Conectando...', `
          <button class="btn-mute" id="mute-btn" title="Silenciar micrófono"><i class="fas fa-microphone"></i></button>
          <button class="btn-hangup" id="hangup-call-btn" title="Colgar"><i class="fas fa-phone-slash"></i></button>
        `);
        
        // Configurar botones de llamada activa
        document.getElementById('mute-btn').addEventListener('click', toggleMute);
        document.getElementById('hangup-call-btn').addEventListener('click', () => {
          // Registrar fin de llamada con duración
          const duration = calculateCallDuration(callStartTime);
          logCallEvent('ended', otherUserId, false, callStartTime.toISOString(), new Date().toISOString(), duration);
          endCurrentCall("Llamada finalizada");
        });
      });
      
      // Evento para rechazar la llamada
      document.getElementById('decline-call-btn').addEventListener('click', () => {
        // Registrar llamada rechazada
        logCallEvent('rejected', otherUserId, false, callStartTime.toISOString());
        call.close();
        hideCallModal();
      });
    }
    
    // Iniciar una llamada saliente
    function initiateCall() {
      // Verificar que no hay una llamada activa
      if (currentCall) {
        alert("Ya estás en una llamada.");
        return;
      }
      
      // Verificar que tenemos acceso al audio
      if (!localCallStream) {
        console.error("No hay acceso al audio para iniciar la llamada");
        
        // Intentar obtener el audio al momento de llamar (para iOS/Safari)
        navigator.mediaDevices.getUserMedia({ audio: true })
          .then(stream => {
            localCallStream = stream;
            continueInitiateCall();
          })
          .catch(err => {
            alert("No se puede iniciar la llamada sin acceso al micrófono");
            console.error("Error al obtener audio para llamada:", err);
          });
        return;
      }
      
      continueInitiateCall();
    }
    
    // Continuar con la llamada una vez que tenemos audio
    function continueInitiateCall() {
      // Registrar inicio de llamada en la base de datos
      callStartTime = new Date();
      logCallEvent('initiated', otherUserId, true, callStartTime.toISOString());
      
      // Mostrar UI de llamada saliente
      showCallModal(`Llamando a ${otherUserName}...`, `
        <button class="btn-cancel" id="cancel-call-btn"><i class="fas fa-times"></i></button>
      `);
      
      // Asegurarse que el peer esté conectado
      if (!peer || !peer.open) {
        console.warn("PeerJS no está conectado. Intentando reconectar...");
        initializePeer();
        setTimeout(continueInitiateCall, 2000);
        return;
      }
      
      try {
        // Intentar realizar la llamada
        const call = peer.call(`user_${otherUserId}`, localCallStream);
        if (!call) {
          throw new Error("No se pudo iniciar la llamada");
        }
        
        currentCall = call;
        setupCallEventHandlers(call);
        
        // Configurar evento para cancelar la llamada
        document.getElementById('cancel-call-btn').addEventListener('click', () => {
          // Registrar llamada cancelada
          logCallEvent('ended', otherUserId, true, callStartTime.toISOString(), new Date().toISOString(), 0);
          endCurrentCall("Llamada cancelada");
        });
        
        // Configurar timeout para la llamada no contestada
        const callTimeout = setTimeout(() => {
          if (currentCall && !callTimerInterval) { // Si no ha iniciado el timer, la llamada no fue contestada
            // Registrar llamada perdida
            logCallEvent('missed', otherUserId, true, callStartTime.toISOString());
            alert("No hubo respuesta a la llamada");
            endCurrentCall("Sin respuesta");
          }
        }, 45000); // 45 segundos para contestar
        
        // Cuando se establece la conexión, actualizar UI
        call.on('stream', () => {
          clearTimeout(callTimeout);
          
          // Registrar llamada contestada
          logCallEvent('answered', otherUserId, true, callStartTime.toISOString());
          
          showCallModal('En llamada', `
            <button class="btn-mute" id="mute-btn" title="Silenciar micrófono"><i class="fas fa-microphone"></i></button>
            <button class="btn-hangup" id="hangup-call-btn" title="Colgar"><i class="fas fa-phone-slash"></i></button>
          `);
          
          // Iniciar cronómetro visible
          startCallTimer();
          
          // Configurar botones de llamada activa
          document.getElementById('mute-btn').addEventListener('click', toggleMute);
          document.getElementById('hangup-call-btn').addEventListener('click', () => {
            // Registrar fin de llamada con duración
            const duration = calculateCallDuration(callStartTime);
            logCallEvent('ended', otherUserId, true, callStartTime.toISOString(), new Date().toISOString(), duration);
            endCurrentCall("Llamada finalizada");
          });
        });
        
      } catch (err) {
        console.error("Error al iniciar la llamada:", err);
        // Registrar error en la llamada
        logCallEvent('ended', otherUserId, true, callStartTime.toISOString(), new Date().toISOString(), 0);
        endCurrentCall("Error al iniciar llamada");
        alert("No se pudo realizar la llamada. Intenta de nuevo.");
      }
    }
    
    // Configurar el botón de llamada
    audioCallBtn.addEventListener('click', initiateCall);
    
    // Inicializar PeerJS al cargar la página
    document.addEventListener('DOMContentLoaded', () => {
      setTimeout(initializePeer, 1000); // Pequeño retraso para asegurar que DOM esté listo
    });
  </script>
</body>
</html>